@@ -2,19 +2,22 @@
 
 import logging
 from typing import List
+from typing import Optional
 from typing import Set
 
+from certbot import configuration
 from certbot import errors
 from certbot import util
 from certbot.compat import filesystem
 from certbot.compat import misc
 from certbot.compat import os
+from certbot.display import ops as display_ops
 from certbot.plugins import util as plug_util
 
 logger = logging.getLogger(__name__)
 
 
-def validate_hooks(config):
+def validate_hooks(config: configuration.NamespaceConfig) -> None:
     """Check hook commands are executable."""
     validate_hook(config.pre_hook, "pre")
     validate_hook(config.post_hook, "post")
@@ -22,7 +25,7 @@ def validate_hooks(config):
     validate_hook(config.renew_hook, "renew")
 
 
-def _prog(shell_cmd):
+def _prog(shell_cmd: str) -> Optional[str]:
     """Extract the program run by a shell command.
 
     :param str shell_cmd: command to be executed
@@ -35,10 +38,11 @@ def _prog(shell_cmd):
         plug_util.path_surgery(shell_cmd)
         if not util.exe_exists(shell_cmd):
             return None
+
     return os.path.basename(shell_cmd)
 
 
-def validate_hook(shell_cmd, hook_name):
+def validate_hook(shell_cmd: str, hook_name: str) -> None:
     """Check that a command provided as a hook is plausibly executable.
 
     :raises .errors.HookCommandNotFound: if the command is not found
@@ -48,15 +52,17 @@ def validate_hook(shell_cmd, hook_name):
         if not _prog(cmd):
             path = os.environ["PATH"]
             if os.path.exists(cmd):
-                msg = "{1}-hook command {0} exists, but is not executable.".format(cmd, hook_name)
+                msg = f"{cmd}-hook command {hook_name} exists, but is not executable."
             else:
-                msg = "Unable to find {2}-hook command {0} in the PATH.\n(PATH is {1})".format(
-                    cmd, path, hook_name)
+                msg = (
+                    f"Unable to find {hook_name}-hook command {cmd} in the PATH.\n(PATH is "
+                    f"{path})\nSee also the --disable-hook-validation option."
+                )
 
             raise errors.HookCommandNotFound(msg)
 
 
-def pre_hook(config):
+def pre_hook(config: configuration.NamespaceConfig) -> None:
     """Run pre-hooks if they exist and haven't already been run.
 
     When Certbot is running with the renew subcommand, this function
@@ -80,7 +86,7 @@ def pre_hook(config):
 executed_pre_hooks: Set[str] = set()
 
 
-def _run_pre_hook_if_necessary(command):
+def _run_pre_hook_if_necessary(command: str) -> None:
     """Run the specified pre-hook if we haven't already.
 
     If we've already run this exact command before, a message is logged
@@ -96,7 +102,7 @@ def _run_pre_hook_if_necessary(command):
         executed_pre_hooks.add(command)
 
 
-def post_hook(config):
+def post_hook(config: configuration.NamespaceConfig) -> None:
     """Run post-hooks if defined.
 
     This function also registers any executables found in
@@ -130,7 +136,7 @@ def post_hook(config):
 post_hooks: List[str] = []
 
 
-def _run_eventually(command):
+def _run_eventually(command: str) -> None:
     """Registers a post-hook to be run eventually.
 
     All commands given to this function will be run exactly once in the
@@ -143,13 +149,14 @@ def _run_eventually(command):
         post_hooks.append(command)
 
 
-def run_saved_post_hooks():
+def run_saved_post_hooks() -> None:
     """Run any post hooks that were saved up in the course of the 'renew' verb"""
     for cmd in post_hooks:
         _run_hook("post-hook", cmd)
 
 
-def deploy_hook(config, domains, lineage_path):
+def deploy_hook(config: configuration.NamespaceConfig, domains: List[str],
+                lineage_path: str) -> None:
     """Run post-issuance hook if defined.
 
     :param configuration.NamespaceConfig config: Certbot settings
@@ -160,10 +167,11 @@ def deploy_hook(config, domains, lineage_path):
     """
     if config.deploy_hook:
         _run_deploy_hook(config.deploy_hook, domains,
-                         lineage_path, config.dry_run)
+                         lineage_path, config.dry_run, config.run_deploy_hooks)
 
 
-def renew_hook(config, domains, lineage_path):
+def renew_hook(config: configuration.NamespaceConfig, domains: List[str],
+               lineage_path: str) -> None:
     """Run post-renewal hooks.
 
     This function runs any hooks found in
@@ -183,7 +191,7 @@ def renew_hook(config, domains, lineage_path):
     executed_dir_hooks = set()
     if config.directory_hooks:
         for hook in list_hooks(config.renewal_deploy_hooks_dir):
-            _run_deploy_hook(hook, domains, lineage_path, config.dry_run)
+            _run_deploy_hook(hook, domains, lineage_path, config.dry_run, config.run_deploy_hooks)
             executed_dir_hooks.add(hook)
 
     if config.renew_hook:
@@ -192,10 +200,11 @@ def renew_hook(config, domains, lineage_path):
                         config.renew_hook)
         else:
             _run_deploy_hook(config.renew_hook, domains,
-                             lineage_path, config.dry_run)
+                             lineage_path, config.dry_run, config.run_deploy_hooks)
 
 
-def _run_deploy_hook(command, domains, lineage_path, dry_run):
+def _run_deploy_hook(command: str, domains: List[str], lineage_path: str, dry_run: bool,
+                     run_deploy_hooks: bool) -> None:
     """Run the specified deploy-hook (if not doing a dry run).
 
     If dry_run is True, command is not run and a message is logged
@@ -207,10 +216,11 @@ def _run_deploy_hook(command, domains, lineage_path, dry_run):
     :type domains: `list` of `str`
     :param str lineage_path: live directory path for the new cert
     :param bool dry_run: True iff Certbot is doing a dry run
+    :param bool run_deploy_hooks: True if deploy hooks should run despite Certbot doing a dry run
 
     """
-    if dry_run:
-        logger.warning("Dry run: skipping deploy hook command: %s",
+    if dry_run and not run_deploy_hooks:
+        logger.info("Dry run: skipping deploy hook command: %s",
                        command)
         return
 
@@ -219,7 +229,7 @@ def _run_deploy_hook(command, domains, lineage_path, dry_run):
     _run_hook("deploy-hook", command)
 
 
-def _run_hook(cmd_name, shell_cmd):
+def _run_hook(cmd_name: str, shell_cmd: str) -> str:
     """Run a hook command.
 
     :param str cmd_name: the user facing name of the hook being run
@@ -227,11 +237,13 @@ def _run_hook(cmd_name, shell_cmd):
     :type shell_cmd: `list` of `str` or `str`
 
     :returns: stderr if there was any"""
-    err, _ = misc.execute_command(cmd_name, shell_cmd, env=util.env_no_snap_for_external_calls())
+    returncode, err, out = misc.execute_command_status(
+        cmd_name, shell_cmd, env=util.env_no_snap_for_external_calls())
+    display_ops.report_executed_command(f"Hook '{cmd_name}'", returncode, out, err)
     return err
 
 
-def list_hooks(dir_path):
+def list_hooks(dir_path: str) -> List[str]:
     """List paths to all hooks found in dir_path in sorted order.
 
     :param str dir_path: directory to search
