@@ -1,10 +1,6 @@
 """Utilities for all Certbot."""
-# distutils.version under virtualenv confuses pylint
-# For more info, see: https://github.com/PyCQA/pylint/issues/73
 import argparse
 import atexit
-import collections
-import distutils.version
 import errno
 import logging
 import platform
@@ -12,12 +8,16 @@ import re
 import socket
 import subprocess
 import sys
+from typing import Any
+from typing import Callable
 from typing import Dict
 from typing import IO
-from typing import Text
+from typing import List
+from typing import NamedTuple
+from typing import Optional
+from typing import Set
 from typing import Tuple
 from typing import Union
-import warnings
 
 import configargparse
 
@@ -34,9 +34,18 @@ if _USE_DISTRO:
 logger = logging.getLogger(__name__)
 
 
-Key = collections.namedtuple("Key", "file pem")
-# Note: form is the type of data, "pem" or "der"
-CSR = collections.namedtuple("CSR", "file data form")
+class Key(NamedTuple):
+    """Container for an optional file path and contents for a PEM-formated private key."""
+    file: Optional[str]
+    pem: bytes
+
+
+class CSR(NamedTuple):
+    """Container for an optional file path and contents for a PEM or DER-formatted CSR."""
+    file: Optional[str]
+    data: bytes
+    # Note: form is the type of data, "pem" or "der"
+    form: str
 
 
 # ANSI SGR escape codes
@@ -61,9 +70,9 @@ _INITIAL_PID = os.getpid()
 # program exits before the lock is cleaned up, it is automatically
 # released, but the file isn't deleted.
 _LOCKS: Dict[str, lock.LockFile] = {}
+_VERSION_COMPONENT_RE = re.compile(r'(\d+ | [a-z]+ | \.)', re.VERBOSE)
 
-
-def env_no_snap_for_external_calls():
+def env_no_snap_for_external_calls() -> Dict[str, str]:
     """
     When Certbot is run inside a Snap, certain environment variables
     are modified. But Certbot sometimes calls out to external programs,
@@ -88,7 +97,7 @@ def env_no_snap_for_external_calls():
     return env
 
 
-def run_script(params, log=logger.error):
+def run_script(params: List[str], log: Callable[[str], None]=logger.error) -> Tuple[str, str]:
     """Run the script with the given params.
 
     :param list params: List of parameters to pass to subprocess.run
@@ -118,7 +127,7 @@ def run_script(params, log=logger.error):
     return proc.stdout, proc.stderr
 
 
-def exe_exists(exe):
+def exe_exists(exe: str) -> bool:
     """Determine whether path/name refers to an executable.
 
     :param str exe: Executable path or name
@@ -137,7 +146,7 @@ def exe_exists(exe):
     return False
 
 
-def lock_dir_until_exit(dir_path):
+def lock_dir_until_exit(dir_path: str) -> None:
     """Lock the directory at dir_path until program exit.
 
     :param str dir_path: path to directory
@@ -152,7 +161,7 @@ def lock_dir_until_exit(dir_path):
         _LOCKS[dir_path] = lock.lock_dir(dir_path)
 
 
-def _release_locks():
+def _release_locks() -> None:
     for dir_lock in _LOCKS.values():
         try:
             dir_lock.release()
@@ -162,7 +171,7 @@ def _release_locks():
     _LOCKS.clear()
 
 
-def set_up_core_dir(directory, mode, strict):
+def set_up_core_dir(directory: str, mode: int, strict: bool) -> None:
     """Ensure directory exists with proper permissions and is locked.
 
     :param str directory: Path to a directory.
@@ -181,7 +190,7 @@ def set_up_core_dir(directory, mode, strict):
         raise errors.Error(PERM_ERR_FMT.format(error))
 
 
-def make_or_verify_dir(directory, mode=0o755, strict=False):
+def make_or_verify_dir(directory: str, mode: int = 0o755, strict: bool = False) -> None:
     """Make sure directory exists with proper permissions.
 
     :param str directory: Path to a directory.
@@ -208,7 +217,7 @@ def make_or_verify_dir(directory, mode=0o755, strict=False):
             raise
 
 
-def safe_open(path: str, mode: str = "w", chmod=None) -> IO:
+def safe_open(path: str, mode: str = "w", chmod: Optional[int] = None) -> IO:
     """Safely open a file.
 
     :param str path: Path to a file.
@@ -225,12 +234,12 @@ def safe_open(path: str, mode: str = "w", chmod=None) -> IO:
     return os.fdopen(fd, mode, *fdopen_args)
 
 
-def _unique_file(path, filename_pat, count, chmod, mode):
+def _unique_file(path: str, filename_pat: Callable[[int], str], count: int,
+                 chmod: int, mode: str) -> Tuple[IO, str]:
     while True:
         current_path = os.path.join(path, filename_pat(count))
         try:
-            return safe_open(current_path, chmod=chmod, mode=mode),\
-                os.path.abspath(current_path)
+            return safe_open(current_path, chmod=chmod, mode=mode), os.path.abspath(current_path)
         except OSError as err:
             # "File exists," is okay, try a different name.
             if err.errno != errno.EEXIST:
@@ -238,7 +247,7 @@ def _unique_file(path, filename_pat, count, chmod, mode):
         count += 1
 
 
-def unique_file(path, chmod=0o777, mode="w"):
+def unique_file(path: str, chmod: int = 0o777, mode: str = "w") -> Tuple[IO, str]:
     """Safely finds a unique file.
 
     :param str path: path/filename.ext
@@ -254,7 +263,8 @@ def unique_file(path, chmod=0o777, mode="w"):
         count=0, chmod=chmod, mode=mode)
 
 
-def unique_lineage_name(path, filename, chmod=0o644, mode="w"):
+def unique_lineage_name(path: str, filename: str, chmod: int = 0o644,
+                        mode: str = "w") -> Tuple[IO, str]:
     """Safely finds a unique file using lineage convention.
 
     :param str path: directory path
@@ -281,7 +291,7 @@ def unique_lineage_name(path, filename, chmod=0o644, mode="w"):
         count=1, chmod=chmod, mode=mode)
 
 
-def safely_remove(path):
+def safely_remove(path: str) -> None:
     """Remove a file that may not exist."""
     try:
         os.remove(path)
@@ -290,7 +300,7 @@ def safely_remove(path):
             raise
 
 
-def get_filtered_names(all_names):
+def get_filtered_names(all_names: Set[str]) -> Set[str]:
     """Removes names that aren't considered valid by Let's Encrypt.
 
     :param set all_names: all names found in the configuration
@@ -307,7 +317,7 @@ def get_filtered_names(all_names):
             logger.debug('Not suggesting name "%s"', name, exc_info=True)
     return filtered_names
 
-def get_os_info():
+def get_os_info() -> Tuple[str, str]:
     """
     Get OS name and version
 
@@ -317,7 +327,7 @@ def get_os_info():
 
     return get_python_os_info(pretty=False)
 
-def get_os_info_ua():
+def get_os_info_ua() -> str:
     """
     Get OS name and version string for User Agent
 
@@ -331,7 +341,7 @@ def get_os_info_ua():
         return " ".join(get_python_os_info(pretty=True))
     return os_info
 
-def get_systemd_os_like():
+def get_systemd_os_like() -> List[str]:
     """
     Get a list of strings that indicate the distribution likeness to
     other distributions.
@@ -344,7 +354,7 @@ def get_systemd_os_like():
         return distro.like().split(" ")
     return []
 
-def get_var_from_file(varname, filepath="/etc/os-release"):
+def get_var_from_file(varname: str, filepath: str = "/etc/os-release") -> str:
     """
     Get single value from a file formatted like systemd /etc/os-release
 
@@ -366,14 +376,14 @@ def get_var_from_file(varname, filepath="/etc/os-release"):
             return _normalize_string(line.strip()[len(var_string):])
     return ""
 
-def _normalize_string(orig):
+def _normalize_string(orig: str) -> str:
     """
     Helper function for get_var_from_file() to remove quotes
     and whitespaces
     """
     return orig.replace('"', '').replace("'", "").strip()
 
-def get_python_os_info(pretty=False):
+def get_python_os_info(pretty: bool = False) -> Tuple[str, str]:
     """
     Get Operating System type/distribution and major version
     using python platform module
@@ -432,7 +442,7 @@ def get_python_os_info(pretty=False):
 EMAIL_REGEX = re.compile("[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+$")
 
 
-def safe_email(email):
+def safe_email(email: str) -> bool:
     """Scrub email address before using it."""
     if EMAIL_REGEX.match(email) is not None:
         return not email.startswith(".") and ".." not in email
@@ -442,11 +452,13 @@ def safe_email(email):
 
 class DeprecatedArgumentAction(argparse.Action):
     """Action to log a warning when an argument is used."""
-    def __call__(self, unused1, unused2, unused3, option_string=None):
-        warnings.warn("Use of %s is deprecated." % option_string, DeprecationWarning)
+    def __call__(self, unused1: Any, unused2: Any, unused3: Any,
+                 option_string: Optional[str] = None) -> None:
+        logger.warning("Use of %s is deprecated.", option_string)
 
 
-def add_deprecated_argument(add_argument, argument_name, nargs):
+def add_deprecated_argument(add_argument: Callable[..., None], argument_name: str,
+                            nargs: Union[str, int]) -> None:
     """Adds a deprecated argument with the name argument_name.
 
     Deprecated arguments are not shown in the help. If they are used on
@@ -472,11 +484,11 @@ def add_deprecated_argument(add_argument, argument_name, nargs):
                  help=argparse.SUPPRESS, nargs=nargs)
 
 
-def enforce_le_validity(domain):
+def enforce_le_validity(domain: str) -> str:
     """Checks that Let's Encrypt will consider domain to be valid.
 
     :param str domain: FQDN to check
-    :type domain: `str` or `unicode`
+    :type domain: `str`
     :returns: The domain cast to `str`, with ASCII-only contents
     :rtype: str
     :raises ConfigurationError: for invalid domains and cases where Let's
@@ -505,12 +517,13 @@ def enforce_le_validity(domain):
                     label, domain))
     return domain
 
-def enforce_domain_sanity(domain):
+
+def enforce_domain_sanity(domain: Union[str, bytes]) -> str:
     """Method which validates domain value and errors out if
     the requirements are not met.
 
     :param domain: Domain to check
-    :type domain: `str` or `unicode`
+    :type domain: `str` or `bytes`
     :raises ConfigurationError: for invalid domains and cases where Let's
                                 Encrypt currently will not issue certificates
 
@@ -529,7 +542,7 @@ def enforce_domain_sanity(domain):
     domain = domain.lower()
 
     # Remove trailing dot
-    domain = domain[:-1] if domain.endswith(u'.') else domain
+    domain = domain[:-1] if domain.endswith('.') else domain
 
     # Separately check for odd "domains" like "http://example.com" to fail
     # fast and provide a clear error message
@@ -564,11 +577,11 @@ def enforce_domain_sanity(domain):
     return domain
 
 
-def is_ipaddress(address):
+def is_ipaddress(address: str) -> bool:
     """Is given address string form of IP(v4 or v6) address?
 
     :param address: address to check
-    :type address: `str` or `unicode`
+    :type address: `str`
 
     :returns: True if address is valid IP address, otherwise return False.
     :rtype: bool
@@ -587,36 +600,22 @@ def is_ipaddress(address):
             return False
 
 
-def is_wildcard_domain(domain):
+def is_wildcard_domain(domain: Union[str, bytes]) -> bool:
     """"Is domain a wildcard domain?
 
     :param domain: domain to check
-    :type domain: `bytes` or `str` or `unicode`
+    :type domain: `bytes` or `str`
 
     :returns: True if domain is a wildcard, otherwise, False
     :rtype: bool
 
     """
-    wildcard_marker: Union[Text, bytes] = b"*."
     if isinstance(domain, str):
-        wildcard_marker = u"*."
-    return domain.startswith(wildcard_marker)
+        return domain.startswith("*.")
+    return domain.startswith(b"*.")
 
 
-def get_strict_version(normalized):
-    """Converts a normalized version to a strict version.
-
-    :param str normalized: normalized version string
-
-    :returns: An equivalent strict version
-    :rtype: distutils.version.StrictVersion
-
-    """
-    # strict version ending with "a" and a number designates a pre-release
-    return distutils.version.StrictVersion(normalized.replace(".dev", "a"))
-
-
-def is_staging(srv):
+def is_staging(srv: str) -> bool:
     """
     Determine whether a given ACME server is a known test / staging server.
 
@@ -627,7 +626,7 @@ def is_staging(srv):
     return srv == constants.STAGING_URI or "staging" in srv
 
 
-def atexit_register(func, *args, **kwargs):
+def atexit_register(func: Callable, *args: Any, **kwargs: Any) -> None:
     """Sets func to be called before the program exits.
 
     Special care is taken to ensure func is only called when the process
@@ -639,6 +638,32 @@ def atexit_register(func, *args, **kwargs):
     atexit.register(_atexit_call, func, *args, **kwargs)
 
 
-def _atexit_call(func, *args, **kwargs):
+def parse_loose_version(version_string: str) -> List[Union[int, str]]:
+    """Parses a version string into its components.
+
+    This code and the returned tuple is based on the now deprecated
+    distutils.version.LooseVersion class from the Python standard library.
+    Two LooseVersion classes and two lists as returned by this function should
+    compare in the same way. See
+    https://github.com/python/cpython/blob/v3.10.0/Lib/distutils/version.py#L205-L347.
+
+    :param str version_string: version string
+
+    :returns: list of parsed version string components
+    :rtype: list
+
+    """
+    components: List[Union[int, str]]
+    components = [x for x in _VERSION_COMPONENT_RE.split(version_string)
+                          if x and x != '.']
+    for i, obj in enumerate(components):
+        try:
+            components[i] = int(obj)
+        except ValueError:
+            pass
+    return components
+
+
+def _atexit_call(func: Callable, *args: Any, **kwargs: Any) -> None:
     if _INITIAL_PID == os.getpid():
         func(*args, **kwargs)
